/*The main game source file for Stick Territory

Copyright 2006 James Stanley*/

#include "game.h"

volatile int counter = 0;
PLAYER *player_list = NULL;

int bluepoints = 0;
char *bluename = "Blue Team";
int redpoints = 0;
char *redname = "Red Team";

BITMAP *quit;
BITMAP *endturn;
BITMAP *help;
BITMAP *menu;

BITMAP *cursor[1];

BITMAP *left;
BITMAP *right;
BITMAP *jump;
BITMAP *done;

int blue;
int red;
int black;
int white;
int darkred;
int darkblue;
int grass;

void game_play(void) {
  //User-oriented:
  int quit = 0;     //Should we quit next time we check?
  int status = 0;   //The current status of the game
  int gamestate = 0;//What the PLYR/AI loop should be doing
  int players = 0;  //How many HUMANS
  int ai = 0;       //How manu AI
  int pointer = 0;  //The cursor to draw
  BITMAP *canvas = create_bitmap(640, 480);
  BITMAP *game = create_bitmap(2000, 380);
  //Framework-oriented:
  int action = 0;       //The action to do, from a return value
  int esc = 0;          //Is ESC pressed?
  int i;                //For the for loops
  int curplayer = 0;    //So we know which to do. Applies to AI and HUMANS
  int offset = 0;       //The offset. For side scrolling... Duh...
  PLAYER *player = NULL;//pointer to the player we're currently using
  PLAYER *tempry;       //A temporary player pointer

  while(!quit) {
    if(counter >= 1) {
      if(key[KEY_F1]) set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0); //Don't need to check this mode because it was set at the start so it will work
      if(key[KEY_F2]) {
        if(set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, 640, 480, 0, 0) != 0) {
          set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0);
        }
      }
                                                   
      switch(status) {
        case MENU://////////////////////////////////////////////
          draw_sprite(canvas, menu, 0, 0);
          textout_centre_ex(canvas, font, "                                                   ", 320, 158, black, white);
          textout_centre_ex(canvas, font, "     - - - S T I C K   T E R R I T O R Y - - -     ", 320, 166, black, white); 
          textout_centre_ex(canvas, font, "                                                   ", 320, 174, black, white);
          textout_centre_ex(canvas, font, "                                                   ", 320, 182, black, white);
          textout_centre_ex(canvas, font, "       Press [ENTER] to play Stick Territory!      ", 320, 190, black, white);
          textout_centre_ex(canvas, font, "                Press [ESC] to exit.               ", 320, 198, black, white);
          textout_centre_ex(canvas, font, "   Press [F1] at any time to enter windowed mode.  ", 320, 206, black, white);
          textout_centre_ex(canvas, font, " Press [F2] at any time to enter full screen mode. ", 320, 214, black, white);
          textout_centre_ex(canvas, font, "                                                   ", 320, 222, black, white);
          action = update_panel(canvas, NOTHIN, player_list);
          if(action == QUIT) return;
          if(action == HELP) {
            //Screw it, we'll do that some other time
          }
          if(action == SKIP) make_menu();
          if(key[KEY_ENTER]) status = SELECT;
          if((esc == 0) && key[KEY_ESC]) quit = 1;
          if(!key[KEY_ESC]) esc = 0;
          break;
        case PLYR://////////////////////////////////////////////
              clear_to_color(game, white);
              clear_bitmap(canvas);

              draw_scene(game);

              for(tempry=player_list; tempry; tempry=tempry->next) {
                draw_player(game,tempry);
                update_player(tempry);
              }

              if(key[KEY_F1]) set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0);
              if(key[KEY_F2]) set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, 640, 480, 0, 0);

              switch(gamestate) {
                case MOVE:
                  action = update_panel(canvas, gamestate, player);
                  if((action == LEFT) || (action == RIGHT)) move_player(player, action);//Action contains the direction
                  if(action == JUMP) jump_player(player);
                  if(((action != LEFT) && (action != RIGHT) && (action != JUMP) && (player->jumpforce == 0)) || player->credit == 0) reset_player(player);
                  offset = 320-player->x;
                  break;
                case WEAPON:
                  break;
                case AIM:
                  break;
                case ACTION:
                  break;
              }
              update_team_points();

              if(action == QUIT) {
                status = MENU;
                break;
              }
              if(action == SKIP) {
                status = NEXT;
                break;
              }
              if(action == HELP) {
                //Screw it. We'll do that in the future
              }

              if(offset > 0) offset = 0;
              if(offset < -1360) offset = -1360;
              draw_sprite(canvas, game, offset, 0);
          break;
        case AI:////////////////////////////////////////////////
          break;
        case NEXT://///////////////////////////////////////////
          curplayer++;
          gamestate = MOVE;
          if(curplayer == (players + ai)) status = PLYR;
          for(player=player_list; player->index != curplayer; player=player->next);//Get the player with index curplayer
          if(player->ai == 1) status = AI; else status = PLYR;
          player->credit = 1000;
          break;
        case SELECT:////////////////////////////////////////////
          game_reset();
          status = SET;
          break;
        case SET://///////////////////////////////////////////////
          players = 4;
          ai = 0;
          for(i = 0; i < players; i++) {
            new_player(100+(i*150), randnum(80)+280, 0, i, BLUETEAM);
          }
          status = NEXT;
          curplayer = -1;//Trust me...
          player = player_list;
          break;
      }
      counter = 0;
      draw_sprite(canvas, cursor[pointer], mouse_x, mouse_y);
      draw_sprite(screen, canvas, 0, 0);
      clear_bitmap(canvas);
    }
  }
  return;
}

void game_init(void) {
  LOCK_VARIABLE(counter);
  LOCK_FUNCTION(timer);
  install_int_ex(timer, MSEC_TO_TIMER(10));
  colours_init();
  //The quit button:
  quit = create_bitmap(40, 16);
  clear_to_color(quit, black);
  rect(quit, 0, 0, 39, 15, red);
  textout_centre_ex(quit, font, "Quit", 20, 4, red, -1);
  //The end turn button:
  endturn = create_bitmap(40, 16);
  clear_to_color(endturn, black);
  rect(endturn, 0, 0, 39, 15, red);
  textout_centre_ex(endturn, font, "Next", 20, 4, red, -1);
  //The help button:
  help = create_bitmap(40, 16);
  clear_to_color(help, black);
  rect(help, 0, 0, 39, 15, red);
  textout_centre_ex(help, font, "Help", 20, 4, red, -1);

  //Walking/Jumping:
  //The left button:
  left = create_bitmap(16, 16);
  clear_to_color(left, black);
  rect(left, 0, 0, 15, 15, red);
  textout_centre_ex(left, font, "<", 8, 4, red, -1);
  //The right button:
  right = create_bitmap(16, 16);
  clear_to_color(right, black);
  rect(right, 0, 0, 15, 15, red);
  textout_centre_ex(right, font, ">", 8, 4, red, -1);
  //The jump button:
  jump = create_bitmap(40, 16);
  clear_to_color(jump, black);
  rect(jump, 0, 0, 39, 15, red);
  textout_centre_ex(jump, font, "Jump", 20, 4, red, -1);
  //The done button:
  done = create_bitmap(40, 16);
  clear_to_color(done, black);
  rect(done, 0, 0, 39, 15, red);
  textout_centre_ex(done, font, "Done", 20, 4, red, -1);

  //The mouse pointers:
  cursor[ARROW] = make_cursor(ARROW);

  //Make the menu screen:
  make_menu();
}

void game_reset(void) {
  del_all_players();
}

void new_player(int x, int y, int ai, int index, int team) {
  PLAYER *tempry;
  PLAYER *player;

  player = malloc(sizeof(PLAYER));
  player->x = x;
  player->y = y;
  player->hp = 100;
  player->points = randnum(50);
  player->ai = ai;
  player->team = team;
  player->index = index;
  player->laa = 30;
  player->lea = 10;
  player->raa = 30;
  player->rea = 10;
  player->lla = 20;
  player->lka = 10;
  player->rla = 20;
  player->rka = 10;
  player->stage = 0;
  player->jumpforce = 0;
  player->credit = 0;
  player->movestage = 0;
  player->jumpstage = -1;
  player->name = "Charlie";
  player->next = NULL;

  if(!player_list) {
    player_list = player;
  }
  else {
    for(tempry=player_list; tempry->next; tempry=tempry->next);//Get the last player
    tempry->next = player;
  }
}

void del_player(PLAYER *player) {
  PLAYER *i;

  if(player == player_list) {
    player_list=player->next;
    free(player);
  }
  else {
    for(i=player_list; i->next != player; i=i->next);//Get the player before the one we delete
    i->next = player->next;//Set the one before the one we delete to point its next to the one after the one we delete
    free(player);//free the memory of the one we delete
  }
}

void del_all_players(void) {
  PLAYER *tempry = NULL;

  while(player_list != NULL) {
    tempry = player_list;
    player_list = player_list->next;
    free(tempry);
  }
}

void draw_scene(BITMAP *bmp) {
  rectfill(bmp, 0, 360, 2000, 380, grass);
}

void draw_player(BITMAP *bmp, PLAYER *player) {
  int a,o,olda,oldo;
  int uparmlength = 20;
  int lowarmlength = 25;
  int upleglength = 25;
  int lowleglength = 35;
  int colour = darkblue;
  switch(player->team) {
    case BLUETEAM:
      colour = darkblue;
      break;
    case REDTEAM:
      colour = darkred;
      break;
  }

  //Data:
  textprintf_centre_ex(bmp, font, (player->x)+15, (player->y)-170, colour, -1, "%s : %d", player->name, player->points);
  rectfill(bmp, (player->x)-36, (player->y)-160, (player->x)+66, (player->y)-150, black);
  rectfill(bmp, (player->x)-35, (player->y)-159, (player->x)-35+(player->hp), (player->y)-151, makecol(255-(player->hp*2.5),(player->hp*2.5),0));
  //Head:
  circle(bmp, (player->x)+15, (player->y)-125, 15, colour);
  //Body:
  vline(bmp, (player->x)+15, (player->y)-110, (player->y)-50, colour);
  //Left upper arm:
  o = sin(deg2rad(player->laa)) * uparmlength;
  a = cos(deg2rad(player->laa)) * uparmlength;
  line(bmp, (player->x)+15, (player->y)-105, (player->x)+15+o, (player->y)-105+a, colour);
  //Left lower arm:
  oldo = o;
  olda = a;
  o = sin(deg2rad(player->laa - player->lea)) * lowarmlength;
  a = cos(deg2rad(player->laa - player->lea)) * lowarmlength;
  line(bmp, (player->x)+15+oldo, (player->y)-105+olda, (player->x)+15+oldo+o, (player->y)-105+olda+a, colour);
  //Right upper arm:
  o = sin(deg2rad(player->raa)) * uparmlength;
  a = cos(deg2rad(player->raa)) * uparmlength;
  line(bmp, (player->x)+15, (player->y)-105, (player->x)+15-o, (player->y)-105+a, colour);
  //Right lower arm:
  oldo = o;
  olda = a;
  o = sin(deg2rad(player->raa - player->rea)) * lowarmlength;
  a = cos(deg2rad(player->raa - player->rea)) * lowarmlength;
  line(bmp, (player->x)+15-oldo, (player->y)-105+olda, (player->x)+15-oldo-o, (player->y)-105+olda+a, colour);
  //Left upper leg:
  o = sin(deg2rad(player->lla)) * upleglength;
  a = cos(deg2rad(player->lla)) * upleglength;
  line(bmp, (player->x)+15, (player->y)-50, (player->x)+15+o, (player->y)-50+a, colour);
  //Left lower leg:
  oldo = o;
  olda = a;
  o = sin(deg2rad(player->lla - player->lka)) * lowleglength;
  a = cos(deg2rad(player->lla - player->lka)) * lowleglength;
  line(bmp, (player->x)+15+oldo, (player->y)-50+olda, (player->x)+15+oldo+o, (player->y)-50+olda+a, colour);
  //Right upper leg:
  o = sin(deg2rad(player->rla)) * upleglength;
  a = cos(deg2rad(player->rla)) * upleglength;
  line(bmp, (player->x)+15, (player->y)-50, (player->x)+15-o, (player->y)-50+a, colour);
  //Right lower leg:
  oldo = o;
  olda = a;
  o = sin(deg2rad(player->rla - player->rka)) * lowleglength;
  a = cos(deg2rad(player->rla - player->rka)) * lowleglength;
  line(bmp, (player->x)+15-oldo, (player->y)-50+olda, (player->x)+15-oldo-o, (player->y)-50+olda+a, colour);
}

void jump_player(PLAYER *player) {
  if(player->jumpforce == 0) {
    player->jumpforce = 200;
    player->jumpstage = 0;
  }
}

void reset_player(PLAYER *player) {
  player->laa = 30;
  player->lea = 10;
  player->raa = 30;
  player->rea = 10;
  player->lla = 20;
  player->lka = 10;
  player->rla = 20;
  player->rka = 10;
}

void move_player(PLAYER *player, int dir) {
  if(player->credit > 0 && player->jumpstage != 0) {
    player->credit--;
    switch(dir) {
      case LEFT://////////////////////////////////
        player->x--;
        switch(player->movestage) {
          case 0:
            player->lea += 2;
            player->laa--;
            player->rea -= 2;
            player->raa++;
            if(player->lka > -10) player->lka--;
            player->rla--;
            player->lka -=5;
            player->lla -=6;
            if(player->lla <= -80) player->movestage = 1;
            break;
          case 1:
            player->lea += 2;
            player->laa--;
            player->rea -= 2;
            player->raa++;
            player->rla--;
            player->lka += 4;
            player->lla += 3;
            if(player->lka >= -10) player->movestage = 2;
            break;
          case 2:
            player->rea += 2;
            player->raa--;
            player->lea -= 2;
            player->laa++;
            if(player->lka > -10) player->lka--;
            player->lla++;
            player->rka += 5;
            player->rla += 6;
            if(player->rla >= 80) player->movestage = 3;
            break;
          case 3:
            player->rea += 2;
            player->raa--;
            player->lea -= 2;
            player->laa++;
            player->lla++;
            player->rka -= 4;
            player->rla -= 3;
            if(player->rka <= 10) player->movestage = 0;
            break;
        }
        break;
      case RIGHT://///////////////////////////////
        player->x++;
        switch(player->movestage) {
          case 0:
            player->lea -= 2;
            player->laa++;
            player->rea += 2;
            player->raa--;
            if(player->rka > -10) player->rka--;
            player->lla--;
            player->rka -=5;
            player->rla -=6;
            if(player->rla <= -80) player->movestage = 1;
            break;
          case 1:
            player->lea -= 2;
            player->laa++;
            player->rea += 2;
            player->raa--;
            player->lla--;
            player->rka += 4;
            player->rla += 3;
            if(player->rka >= -10) player->movestage = 2;
            break;
          case 2:
            player->rea -= 2;
            player->raa++;
            player->lea += 2;
            player->laa--;
            if(player->rka > -10) player->rka--;
            player->rla++;
            player->lka += 5;
            player->lla += 6;
            if(player->lla >= 80) player->movestage = 3;
            break;
          case 3:
            player->rea -= 2;
            player->raa++;
            player->lea += 2;
            player->laa--;
            player->rla++;
            player->lka -= 4;
            player->lla -= 3;
            if(player->lka <= 10) player->movestage = 0;
            break;
        }
      break;
    }
  }
}

void update_player(PLAYER *player) {
  int o, a, oldo, olda;
  int y1, y2;
  int upleglength = 25;
  int lowleglength = 35;

  //Calculate left leg y:
  oldo = sin(deg2rad(player->lla)) * upleglength;
  olda = cos(deg2rad(player->lla)) * upleglength;
  o = sin(deg2rad(player->lla - player->lka)) * lowleglength;
  a = cos(deg2rad(player->lla - player->lka)) * lowleglength;
  y1 = (player->y)-50+olda+a;
  //Calculate right leg y:
  oldo = sin(deg2rad(player->rla)) * upleglength;
  olda = cos(deg2rad(player->rla)) * upleglength;
  o = sin(deg2rad(player->rla - player->rka)) * lowleglength;
  a = cos(deg2rad(player->rla - player->rka)) * lowleglength;
  y2 = (player->y)-50+olda+a;

  if(y1 > y2) {
    if(y1 > 360) {
      player->y -= 5;
    }
    else {
      if(y1 < 350) {
        player->y += 3;
      }
      if(y1 < 360) {
        player->y++;
      }
    }
  }
  else {
    if(y2 > 360) {
      player->y -= 5;
    }
    else {
      if(y2 < 350) {
        player->y += 3;
      }
      if(y2 < 360) {
        player->y++;
      }
    }
  }

  if((player->jumpforce > 0) && (player->credit > 0)) {
    switch(player->jumpstage) {
      case 0:
        player->rla += 4;
        player->lla += 4;
        player->rka += 6;
        player->lka += 6;
        if(player->rka >= 120) player->jumpstage = 1;
        break;
      case 1:
        player->rla -= 7;
        player->lla -= 7;
        player->rka -= 12;
        player->lka -= 12;
        if(player->rka <= 10) { player->jumpstage = 2; reset_player(player); }
        break;
      case 2:
        player->y -= (player->jumpforce / 20);
        player->jumpforce--;
        player->credit--;
        break;
    }
  }
}

int update_panel(BITMAP *bmp, int gamestate, PLAYER *player) {
  static int clicked;
  int colour;
  int points[3] = {0, 0, 0};
  char *names[3] = {"Nobody", "Nobody", "Nobody"};
  int teams[3] = {BLUETEAM, BLUETEAM, BLUETEAM};
  static float x1, x2, dir1, dir2;
  PLAYER *tempry;

  for(tempry=player_list; tempry; tempry=tempry->next) {
    if(tempry->points > points[0]) {
      points[2] = points[1];
      names[2] = names[1];
      points[1] = points[0];
      names[1] = names[0];
      points[0] = tempry->points;
      names[0] = tempry->name;
      goto next;
    }
    if(tempry->points > points[1]) {
      points[2] = points[1];
      names[2] = names[1];
      points[1] = tempry->points;
      names[1] = tempry->name;
      goto next;
      goto next;
    }
    if(tempry->points > points[2]) {
      points[2] = tempry->points;
      names[2] = tempry->name;
      goto next;
    }
    next:;
  }

  if(bluepoints > redpoints) {
    textprintf_ex(bmp, font, 4, 384, blue, -1, "%04d %s", bluepoints, bluename);
    textprintf_ex(bmp, font, 4, 400, red, -1, "%04d %s", redpoints, redname);
  }
  else {
    textprintf_ex(bmp, font, 4, 384, red, -1, "%04d %s", redpoints, redname);
    textprintf_ex(bmp, font, 4, 400, blue, -1, "%04d %s", bluepoints, bluename);
  }

  hline(bmp, 4, 420, 140, white);

  if(teams[0] == BLUETEAM) colour = blue; else colour = red;
  textprintf_ex(bmp, font, 4, 432, colour, -1, "%04d %s", points[0], names[0]);

  if(teams[1] == BLUETEAM) colour = blue; else colour = red;
  textprintf_ex(bmp, font, 4, 448, colour, -1, "%04d %s", points[1], names[1]);

  if(teams[2] == BLUETEAM) colour = blue; else colour = red;
  textprintf_ex(bmp, font, 4, 464, colour, -1, "%04d %s", points[2], names[2]);

  vline(bmp, 148, 384, 476, white);

  switch(gamestate) {
    case MOVE:
      switch(player->team) {
        case BLUETEAM:
          textprintf_ex(bmp, font, 156, 386, blue, -1, "Current player: %s", player->name);
          break;
        case REDTEAM:
          textprintf_ex(bmp, font, 156, 386, red, -1, "Current player: %s", player->name);
          break;
      }
      draw_sprite(bmp, left, 156, 400);
      draw_sprite(bmp, right, 281, 400);
      draw_sprite(bmp, jump, 301, 400);
      draw_sprite(bmp, done, 345, 400);
      rect(bmp, 176, 400, 276, 415, red);
      rectfill(bmp, 176, 400, (176+(player->credit/10)), 415, red);
      break;
    case AIM:
      break;
    case WEAPON:
      break;
    case ACTION:
      break;
    case NOTHIN:
      break;
  }

  vline(bmp, 588, 384, 440, white);
  hline(bmp, 156, 448, 636, white);

  draw_sprite(bmp, quit, 596, 384);
  draw_sprite(bmp, endturn, 596, 404);
  draw_sprite(bmp, help, 596, 424);

  if(x1 < 156) {
    x1 = 156;
    dir1 = RIGHT;
  }
  if(x2 < 156) {
    x2 = 156;
    dir2 = RIGHT;
  }

  if(x1 > 277) {
    x1 = 277;
    dir1 = LEFT;
  }
  if(x2 > 225) {
    x2 = 220;
    dir2 = LEFT;
  }

  if(dir1 == RIGHT) x1+=1; else x1-=1;
  if(dir2 == RIGHT) x2+=1; else x2-=1;

  textout_ex(bmp, font, "Stick Territory was written by James Stanley.", x1, 456, red, -1);
  textout_ex(bmp, font, "http://ssp.shorturl.com. jamesstanley@bluebottle.com", x2, 466, red, -1);

  if(clicked == 0) {
    clicked = 1;
    if(click(596, 384, 636, 400)) return QUIT;
    if(click(596, 404, 636, 420)) return SKIP;
    if(click(596, 424, 636, 440)) return HELP;
  }

  switch(gamestate) {
    case MOVE:
      if((click(156, 400, 172, 416)) || key[KEY_LEFT]) return LEFT;
      if((click(281, 400, 296, 416)) || key[KEY_RIGHT]) return RIGHT;
      if((click(301, 400, 341, 416)) || key[KEY_SPACE]) return JUMP;
      break;
  }

  if(!(mouse_b & 1)) clicked = 0;
  return 0;
}

void update_team_points(void) {
  PLAYER *player;

  bluepoints = 0;
  redpoints = 0;

  for(player=player_list; player; player=player->next) {
    switch(player->team) {
      case BLUETEAM:
        bluepoints += player->points;
        break;
      case REDTEAM:
        redpoints += player->points;
        break;
    }
  }
}

int click(int x1, int y1, int x2, int y2) {
  if((mouse_x >= x1) && (mouse_x <= x2) && (mouse_y >= y1) && (mouse_y <= y2) && (mouse_b & 1)) return 1; else return 0;
}

int randnum(int num) {//From 0 ... num
  return rand()/(((unsigned)RAND_MAX + 1) / (num+1));
}

void colours_init(void) {
  blue = makecol(0, 0, 255);
  red = makecol(255, 0, 0);
  black = makecol(0, 0, 0);
  white = makecol(255, 255, 255);
  darkred = makecol(128, 0, 0);
  darkblue = makecol(0, 0, 128);
  grass = makecol(0, 128, 0);
}

float rad2deg(float rad) {
  return rad*(180/M_PI);
}

float deg2rad(float deg) {
  return deg*(M_PI/180);
}

void make_menu(void) {
  int x, y;
  int r, g, b;
  int rd, gd, bd;
  int inc;

  menu = create_bitmap(640, 380);
  r = randnum(255);
  rd = UP;
  g = randnum(255);
  gd = UP;
  b = randnum(255);
  bd = UP;
  inc = randnum(20);
  for(x = 0; x < 640; x++) {
    for(y = 0; y < 380; y++) {
      putpixel(menu, x, y, makecol(r,g,b));
    }
    if(rd == UP) r += inc; else r -= inc;
    if(r > 250 || r < 5) {
         switch(rd) {
           case UP:
             rd = DOWN;
             break;
           case DOWN:
             rd = UP;
             break;
         }
    }
    if(gd == UP) g += inc; else g -= inc;
    if(g > 250 || g < 5) {
         switch(gd) {
           case UP:
             gd = DOWN;
             break;
           case DOWN:
             gd = UP;
             break;
         }
    }
    if(bd == UP) b += inc; else b -= inc;
    if(b > 250 || b < 5) {
         switch(bd) {
           case UP:
             bd = DOWN;
             break;
           case DOWN:
             bd = UP;
             break;
         }
    }
  }
}

void timer(void) {
  counter++;
} END_OF_FUNCTION(timer)
