/*Copyright 2006 James Stanley*/

#ifndef GAME_H_INC
#define GAME_H_INC

#include "main.h"
#include "cursor.h"

#define BLUETEAM 1
#define REDTEAM  2

#define MENU   0
#define PLYR   1
#define AI     2
#define NEXT   3
#define SELECT 4
#define SET    5

#define MOVE   0
#define WEAPON 1
#define AIM    2
#define ACTION 3
#define NOTHIN 4

#define WINDOW 0
#define FULL   1

#define ARROW  0

#define QUIT   1
#define SKIP   2
#define HELP   3
#define LEFT   4
#define RIGHT  5
#define JUMP   6
#define UP     7
#define DOWN   8
#define GUN    9
#define BOMB   10

typedef struct PLAYER {
  int x, y;     //Position on the map. Y is somewhere near the bottom of the body
  int hp;       //Hit Points
  int points;   //Total Points
  int ai;       //Artificial 'intelligence'?
  int team;     //What team are they on?
  int index;    //The index (0, 1, 2, 3)
  int laa;      //Left Arm Angle
  int lea;      //Left Elbow Angle
  int raa;      //Right Arm Angle
  int rea;      //Right Elbow Angle
  int lla;      //Left Leg Angle
  int lka;      //Left Knee Angle
  int rla;      //Right Leg Angle
  int rka;      //Right Knee Angle
  int stage;    //The stage of movement they are in
  int jumpforce;//The force they have
  int credit;   //How much movement credit they have
  int movestage;//The stage they are in in their movement
  int jumpstage;//The stage they are in in their flight
  char *name;   //The players name
  struct PLAYER *next;
} PLAYER;

extern volatile int counter;
extern PLAYER *player_list;

extern int bluepoints;
extern char *bluename;
extern int redpoints;
extern char *redname;

extern BITMAP *quit;
extern BITMAP *endturn;
extern BITMAP *help;
extern BITMAP *menu;

extern BITMAP *cursor[1];

extern BITMAP *left;
extern BITMAP *right;
extern BITMAP *jump;
extern BITMAP *done;

extern int blue;
extern int red;
extern int black;
extern int white;
extern int darkred;
extern int darkblue;
extern int grass;

void game_play(void);
void game_init(void);
void game_reset(void);
void new_player(int x, int y, int ai, int index, int team);
void del_player(PLAYER *player);
void del_all_players(void);
void draw_scene(BITMAP *bmp);
void draw_player(BITMAP *bmp, PLAYER *player);
void jump_player(PLAYER *player);
void reset_player(PLAYER *player);
void move_player(PLAYER *player, int dir);
void update_player(PLAYER *player);
int update_panel(BITMAP *bmp, int gamestate, PLAYER *player);
void update_team_points(void);
int click(int x1, int y1, int x2, int y2);
int randnum(int num);
void colours_init(void);
float rad2deg(float rad);
float deg2rad(float deg);
void make_menu(void);
void timer(void);

#endif
